@@ -1,4 +1,3 @@
-
 from functools import partial
 import json
 import time
@@ -6,6 +5,7 @@ import time
 from shakenfist import constants
 from shakenfist import db
 from shakenfist import etcd
+from shakenfist import eventlog
 from shakenfist import exceptions
 from shakenfist import logutil
 from shakenfist.util import general as util_general
@@ -17,13 +17,14 @@ LOG, _ = logutil.setup(__name__)
 class DatabaseBackedObject(object):
     object_type = 'unknown'
     current_version = None
+    upgrade_supported = False
     state_targets = None
 
     STATE_INITIAL = 'initial'
     STATE_CREATING = 'creating'
     STATE_CREATED = 'created'
     STATE_DELETED = 'deleted'
-    STATE_DELETE_WAIT = 'delete_wait'
+    STATE_DELETE_WAIT = 'delete-wait'
     STATE_ERROR = 'error'
 
     ACTIVE_STATES = set([STATE_INITIAL,
@@ -57,10 +58,12 @@ class DatabaseBackedObject(object):
     def unique_label(self):
         return (self.object_type, self.__uuid)
 
-    def add_event(self, operation, phase, duration=None, msg=None):
+    # Shim to track what hasn't been converted to the new style yet
+    def add_event2(self, message, duration=None, extra=None, suppress_event_logging=False):
         if not self.__in_memory_only:
-            db.add_event(
-                self.object_type, self.__uuid, operation, phase, duration, msg)
+            eventlog.add_event2(
+                self.object_type, self.__uuid, message, duration=duration,
+                extra=extra, suppress_event_logging=suppress_event_logging)
 
     @classmethod
     def from_db(cls, object_uuid):
@@ -104,25 +107,21 @@ class DatabaseBackedObject(object):
 
     @classmethod
     def _db_create(cls, object_uuid, metadata):
-        LOG.with_fields(metadata).with_field(
-            cls.object_type, object_uuid).debug('Object created')
         metadata['uuid'] = object_uuid
         etcd.create(cls.object_type, None, object_uuid, metadata)
-
-        db.add_event(
-            cls.object_type, object_uuid, 'db record created', None, None, None)
+        eventlog.add_event2(cls.object_type, object_uuid, 'db record created',
+                            extra=metadata)
 
     @classmethod
     def _db_get(cls, object_uuid):
-        # NOTE(mikal): we don't do upgrades inflight. They are assumed to have
-        # been done as part of the upgrade process.
         o = etcd.get(cls.object_type, None, object_uuid)
         if not o:
             return None
 
         if o.get('version', 0) != cls.current_version:
-            raise exceptions.BadObjectVersion(
-                'Unknown version - %s: %s' % (cls.object_type, o))
+            if not cls.upgrade_supported:
+                raise exceptions.BadObjectVersion(
+                    'Unsupported object version - %s: %s' % (cls.object_type, o))
         return o
 
     # We need to force in memory values through JSON because some values require
@@ -148,6 +147,22 @@ class DatabaseBackedObject(object):
                 yield key, data
 
     def _db_set_attribute(self, attribute, value):
+        # Some attributes are simply too frequently changed to have much meaning
+        # as an event.
+        if (self.object_type, attribute) not in [('node', 'blobs'), ('node', 'observed')]:
+            # Coerce the value into a dictionary.
+            if type(value) is State:
+                event_values = value.obj_dict()
+            elif type(value) is dict:
+                event_values = value.copy()
+            else:
+                event_values = {'value': value}
+
+            # Add the attribute we're setting to the event so we're not confused
+            # later.
+            event_values['attribute'] = attribute
+            self.add_event2('set attribute', extra=event_values)
+
         if self.__in_memory_only:
             self.__in_memory_values[attribute] = json.dumps(
                 value, indent=4, sort_keys=True, cls=etcd.JSONEncoderCustomTypes)
@@ -159,8 +174,28 @@ class DatabaseBackedObject(object):
         if self.__in_memory_only and attribute in self.__in_memory_values:
             del self.__in_memory_values[attribute]
         else:
-            etcd.delete('attribute/%s' %
-                        self.object_type, self.__uuid, attribute)
+            etcd.delete(
+                'attribute/%s' % self.object_type, self.__uuid, attribute)
+
+    def _add_item_in_attribute_list(self, listname, item):
+        with self.get_lock_attr(listname, 'Add %s' % listname):
+            items = self._db_get_attribute(listname).get(listname, [])
+            if item not in items:
+                items.append(item)
+                self._db_set_attribute(listname, {
+                    listname: items,
+                    'initialized': True
+                })
+
+    def _remove_item_in_attribute_list(self, listname, item):
+        with self.get_lock_attr(listname, 'Remove %s' % listname):
+            items = self._db_get_attribute(listname).get(listname, [])
+            if item in items:
+                items.remove(item)
+                self._db_set_attribute(listname, {
+                    listname: items,
+                    'initialized': True
+                })
 
     def get_lock(self, subtype=None, ttl=60, relatedobjects=None, log_ctx=None,
                  op=None, timeout=constants.ETCD_ATTEMPT_TIMEOUT):
@@ -214,9 +249,6 @@ class DatabaseBackedObject(object):
 
             new_state = State(new_value, time.time())
             self._db_set_attribute('state', new_state)
-            self.add_event('state changed',
-                           '%s -> %s' % (orig.value, new_value))
-            self.error = None
 
     @property
     def error(self):
@@ -235,6 +267,12 @@ class DatabaseBackedObject(object):
                     % (s, self.object_type))
         self._db_set_attribute('error', {'message': msg})
 
+    def hard_delete(self):
+        etcd.delete(self.object_type, None, self.uuid)
+        etcd.delete_all('attribute/%s' % self.object_type, self.uuid)
+        db.delete_metadata(self.object_type, self.uuid)
+        self.add_event2('hard deleted object')
+
 
 class DatabaseBackedObjectIterator(object):
     def __init__(self, filters):
@@ -254,7 +292,8 @@ def state_filter(states, o):
 
 # Do not use these filters for instances or nodes, use the more
 # specific ones instead
-active_states_filter = partial(state_filter, DatabaseBackedObject.ACTIVE_STATES)
+active_states_filter = partial(
+    state_filter, DatabaseBackedObject.ACTIVE_STATES)
 
 
 def state_age_filter(delay, o):
