@@ -146,39 +146,9 @@ class SFConfig(BaseSettings):
         description='A URL listing all Cirros releases'
     )
 
-    DOWNLOAD_URL_DEBIAN: AnyHttpUrl = Field(
-        (
-            'https://cloud.debian.org/images/cloud/OpenStack/current-%(vernum)s/'
-            'debian-%(vernum)s-openstack-amd64.qcow2'
-        ),
-        description='Default download mirror for Debian images'
-    )
-    CHECKSUM_URL_DEBIAN: AnyHttpUrl = Field(
-        (
-            'https://cloud.debian.org/images/cloud/OpenStack/'
-            'current-%(vernum)s/MD5SUMS'
-        ),
-        description='Default checksum URL for Debian images'
-    )
-
-    DOWNLOAD_URL_UBUNTU: AnyHttpUrl = Field(
-        (
-            'https://cloud-images.ubuntu.com/%(vername)s/current/'
-            '%(vername)s-server-cloudimg-amd64.img'
-        ),
-        description='Default download mirror for Ubuntu image'
-    )
-    CHECKSUM_URL_UBUNTU: AnyHttpUrl = Field(
-        (
-            'https://cloud-images.ubuntu.com/%(vername)s/current/MD5SUMS'
-        ),
-        description='Default checksum URL for Debian images'
-    )
-    LISTING_URL_UBUNTU: AnyHttpUrl = Field(
-        (
-            'https://cloud-images.ubuntu.com'
-        ),
-        description='A URL listing all Ubuntu releases'
+    IMAGE_DOWNLOAD_URL: AnyHttpUrl = Field(
+        'https://images.shakenfist.com',
+        description='Default download server for Shaken Fist images'
     )
 
     MAX_IMAGE_TRANSFER_SECONDS: int = Field(
@@ -189,18 +159,37 @@ class SFConfig(BaseSettings):
         True, description='Compress snapshots taken of instances'
     )
 
+    SNAPSHOTS_DEFAULT_TO_THIN: bool = Field(
+        False, description='Whether snapshots are thin (just changes from base image) or thick'
+    )
+
     # Artifact options
     ARTIFACT_MAX_VERSIONS_DEFAULT: int = Field(
         3, description='Default number of versions to keep within an artifact'
     )
 
-    # Other options
+    # Event options
+    EVENTLOG_NODE_IP: str = Field(
+        '', description='Mesh IP of the node which stores event logs',
+    )
+    USAGE_EVENT_FREQUENCY: int = Field(
+        60, description='How frequently to collect usage events.'
+    )
+
+    # Blob options
     BLOB_REPLICATION_FACTOR: int = Field(
         2, description='How many copies of each blob we like to have.'
     )
     MAX_CONCURRENT_BLOB_TRANSFERS: int = Field(
         20, description='How many concurrent blob transfers we can have queued.'
     )
+    BLOB_TRANSCODE_MAXIMUM_IDLE_TIME: int = Field(
+        24 * 3600,
+        description=('How long we keep a unused cached transcode of a blob '
+                     'before reaping.')
+    )
+
+    # Other options
     ZONE: str = Field(
         'shakenfist', description='What nova called an availability zone'
     )
@@ -220,6 +209,9 @@ class SFConfig(BaseSettings):
     NODE_IS_NETWORK_NODE: bool = Field(
         False, description='True if this node is the network node.'
     )
+    NODE_IS_EVENTLOG_NODE: bool = Field(
+        False, description='True if this node is a the eventlog node.'
+    )
 
     DISK_BUS: str = Field(
         'virtio',
@@ -251,6 +243,9 @@ class SFConfig(BaseSettings):
     LIBVIRT_GROUP: str = Field(
         'libvirt-qemu', description='Name of the libvirt group'
     )
+    LIBVIRT_LOG_PATH: str = Field(
+        '/var/log/libvirt/qemu', description='Path to libvirt logs'
+    )
 
     ENABLE_EVENTS: bool = Field(
         True, description='Record events about objects in the database. '
@@ -264,12 +259,16 @@ class SFConfig(BaseSettings):
     LOGLEVEL_MAIN: str = 'info'
     LOGLEVEL_NET: str = 'info'
     LOGLEVEL_RESOURCES: str = 'info'
-    LOGLEVEL_TRIGGERS: str = 'info'
+    LOGLEVEL_SIDECHANNEL: str = 'info'
     LOGLEVEL_QUEUES: str = 'info'
     LOG_METHOD_TRACE: bool = Field(
         False, description='Add method name and module line number to log messages'
     )
 
+    # etcd
+    ETCD_HOST: str = Field(
+        'localhost', description='Hostname or IP of the etcd host to query'
+    )
     EXCESSIVE_ETCD_CACHE_LOGGING: bool = Field(
         False, description='Record detailed information about etcd cache performance.'
     )
