@@ -1,26 +1,21 @@
 # Copyright 2019 Michael Still
 
-import setproctitle
-import time
+import faulthandler
 import os
 import psutil
+import setproctitle
 import signal
-import sys
+import subprocess
+import time
 
 from shakenfist.config import config
 from shakenfist.daemons import daemon
-from shakenfist.daemons import external_api as external_api_daemon
-from shakenfist.daemons import cleaner as cleaner_daemon
-from shakenfist.daemons import cluster as cluster_daemon
-from shakenfist.daemons import queues as queues_daemon
-from shakenfist.daemons import net as net_daemon
-from shakenfist.daemons import resources as resource_daemon
-from shakenfist.daemons import triggers as trigger_daemon
+from shakenfist.daemons import shim
 from shakenfist import etcd
 from shakenfist import instance
 from shakenfist.ipmanager import IPManager
 from shakenfist import logutil
-from shakenfist import net
+from shakenfist import network
 from shakenfist.networkinterface import interfaces_for_instance
 from shakenfist.node import Node
 from shakenfist.util import general as util_general
@@ -74,53 +69,53 @@ def restore_instances():
         else:
             instances.append(inst)
 
-    with util_general.RecordedOperation('restore networks', None):
-        for network in networks:
-            try:
-                n = net.Network.from_db(network)
-                if not n.is_dead():
-                    LOG.with_object(n).info('Restoring network')
-                    n.create_on_hypervisor()
-                    n.ensure_mesh()
-            except Exception as e:
-                util_general.ignore_exception(
-                    'restore network %s' % network, e)
+    for network_uuid in networks:
+        try:
+            n = network.Network.from_db(network_uuid)
+            if not n.is_dead():
+                LOG.with_object(n).info('Restoring network')
+                n.create_on_hypervisor()
+                n.ensure_mesh()
+        except Exception as e:
+            util_general.ignore_exception(
+                'restore network %s' % network_uuid, e)
 
-    with util_general.RecordedOperation('restore instances', None):
-        for inst in instances:
-            try:
-                with inst.get_lock(ttl=120, timeout=120, op='Instance restore'):
-                    started = ['on', 'transition-to-on',
-                               instance.Instance.STATE_INITIAL, 'unknown']
-                    if inst.power_state not in started:
-                        continue
+    for inst in instances:
+        try:
+            with inst.get_lock(ttl=120, timeout=120, op='Instance restore'):
+                started = ['on', 'transition-to-on',
+                           instance.Instance.STATE_INITIAL, 'unknown']
+                if inst.power_state not in started:
+                    continue
 
-                    LOG.with_object(inst).info('Restoring instance')
-                    inst.create_on_hypervisor()
-            except Exception as e:
-                util_general.ignore_exception(
-                    'restore instance %s' % inst, e)
-                inst.etcd.enqueue_delete_due_error(
-                    'exception while restoring instance on daemon restart')
+                LOG.with_object(inst).info('Restoring instance')
+                inst.create_on_hypervisor()
+        except Exception as e:
+            util_general.ignore_exception(
+                'restore instance %s' % inst, e)
+            inst.etcd.enqueue_delete_due_error(
+                'exception while restoring instance on daemon restart')
 
 
-DAEMON_IMPLEMENTATIONS = {
-    'api': external_api_daemon,
-    'cleaner': cleaner_daemon,
-    'cluster': cluster_daemon,
-    'net': net_daemon,
-    'queues': queues_daemon,
-    'resources': resource_daemon,
-    'triggers': trigger_daemon
-}
+DAEMON_PROCS = {}
 
 
-DAEMON_PIDS = {}
+def emit_trace():
+    # We have a bunch of subprocesses here, so we can't just use the default
+    # faulthandler mechanism.
+    faulthandler.dump_traceback
+    for proc in DAEMON_PROCS:
+        try:
+            os.kill(DAEMON_PROCS[proc].pid, signal.SIGUSR1)
+        except ProcessLookupError:
+            pass
+
+
+signal.signal(signal.SIGUSR1, emit_trace)
 
 
 def main():
-    global DAEMON_IMPLEMENTATIONS
-    global DAEMON_PIDS
+    global DAEMON_PROCS
 
     LOG.info('Starting...')
     setproctitle.setproctitle(
@@ -142,17 +137,9 @@ def main():
     etcd.restart_queues()
 
     def _start_daemon(d):
-        pid = os.fork()
-        if pid == 0:
-            try:
-                DAEMON_IMPLEMENTATIONS[d].Monitor(d).run()
-                sys.exit(0)
-            except Exception as e:
-                util_general.ignore_exception('daemon creation', e)
-                sys.exit(1)
-
-        DAEMON_PIDS[pid] = d
-        LOG.with_field('pid', pid).info('Started %s' % d)
+        DAEMON_PROCS[d] = subprocess.Popen(
+            ['/srv/shakenfist/venv/bin/sf-daemon-shim', d])
+        LOG.with_field('pid', DAEMON_PROCS[d].pid).info('Started %s' % d)
 
     # Resource usage publisher, we need this early because scheduling decisions
     # might happen quite early on.
@@ -161,9 +148,9 @@ def main():
     # If I am the network node, I need some setup
     if config.NODE_IS_NETWORK_NODE:
         # Bootstrap the floating network in the Networks table
-        floating_network = net.Network.from_db('floating')
+        floating_network = network.Network.from_db('floating')
         if not floating_network:
-            floating_network = net.Network.create_floating_network(
+            floating_network = network.Network.create_floating_network(
                 config.FLOATING_NETWORK)
 
         subst = {
@@ -179,64 +166,71 @@ def main():
             # all we do is create a bridge if it doesn't exist and the wire
             # everything up to it. We can do egress NAT in that state, even if
             # floating IPs don't work.
-            with util_general.RecordedOperation('create physical bridge', None):
-                # No locking as read only
-                ipm = IPManager.from_db('floating')
-                subst['master_float'] = ipm.get_address_at_index(1)
-                subst['netmask'] = ipm.netmask
+            #
+            # No locking as read only
+            ipm = IPManager.from_db('floating')
+            subst['master_float'] = ipm.get_address_at_index(1)
+            subst['netmask'] = ipm.netmask
 
-                # We need to copy the MTU of the interface we are bridging to
-                # or weird networking things happen.
-                mtu = util_network.get_interface_mtu(config.NODE_EGRESS_NIC)
+            # We need to copy the MTU of the interface we are bridging to
+            # or weird networking things happen.
+            mtu = util_network.get_interface_mtu(config.NODE_EGRESS_NIC)
 
-                util_network.create_interface(
-                    subst['egress_bridge'], 'bridge', '', mtu=mtu)
+            util_network.create_interface(
+                subst['egress_bridge'], 'bridge', '', mtu=mtu)
 
-                util_process.execute(None,
-                                     'ip link set %(egress_bridge)s up' % subst)
-                util_process.execute(None,
-                                     'ip addr add %(master_float)s/%(netmask)s '
-                                     'dev %(egress_bridge)s' % subst)
+            util_process.execute(None,
+                                 'ip link set %(egress_bridge)s up' % subst)
+            util_process.execute(None,
+                                 'ip addr add %(master_float)s/%(netmask)s '
+                                 'dev %(egress_bridge)s' % subst)
 
-                util_process.execute(None,
-                                     'iptables -A FORWARD -o %(egress_nic)s '
-                                     '-i %(egress_bridge)s -j ACCEPT' % subst)
-                util_process.execute(None,
-                                     'iptables -A FORWARD -i %(egress_nic)s '
-                                     '-o %(egress_bridge)s -j ACCEPT' % subst)
-                util_process.execute(None,
-                                     'iptables -t nat -A POSTROUTING '
-                                     '-o %(egress_nic)s -j MASQUERADE' % subst)
+            util_process.execute(None,
+                                 'iptables -A FORWARD -o %(egress_nic)s '
+                                 '-i %(egress_bridge)s -j ACCEPT' % subst)
+            util_process.execute(None,
+                                 'iptables -A FORWARD -i %(egress_nic)s '
+                                 '-o %(egress_bridge)s -j ACCEPT' % subst)
+            util_process.execute(None,
+                                 'iptables -t nat -A POSTROUTING '
+                                 '-o %(egress_nic)s -j MASQUERADE' % subst)
 
     def _audit_daemons():
         running_daemons = []
-        for pid in DAEMON_PIDS:
-            running_daemons.append(DAEMON_PIDS[pid])
+        for proc in DAEMON_PROCS:
+            running_daemons.append(proc)
 
-        for d in DAEMON_IMPLEMENTATIONS:
+        for d in shim.DAEMON_IMPLEMENTATIONS:
             if d not in running_daemons:
                 _start_daemon(d)
 
-        for d in list(DAEMON_PIDS):
-            if not psutil.pid_exists(d):
-                LOG.warning('%s pid is missing, restarting' % DAEMON_PIDS[d])
-                _start_daemon(DAEMON_PIDS[d])
+        for proc in list(DAEMON_PROCS):
+            if not psutil.pid_exists(DAEMON_PROCS[proc].pid):
+                LOG.warning('%s pid is missing, restarting' % DAEMON_PROCS[d])
+                del DAEMON_PROCS[d]
+                _start_daemon(DAEMON_PROCS[d])
+
+    if not config.NODE_IS_EVENTLOG_NODE:
+        del shim.DAEMON_IMPLEMENTATIONS['eventlog']
 
     _audit_daemons()
     restore_instances()
 
     running = True
+    shutdown_commenced = None
     while True:
         time.sleep(5)
 
         try:
-            wpid, _ = os.waitpid(-1, os.WNOHANG)
-            while wpid != 0:
-                LOG.warning('%s exited (pid %d)'
-                            % (DAEMON_PIDS.get(wpid, 'unknown'), wpid))
-                if wpid in DAEMON_PIDS:
-                    del DAEMON_PIDS[wpid]
-                wpid, _ = os.waitpid(-1, os.WNOHANG)
+            dead = []
+            for proc in DAEMON_PROCS:
+                if DAEMON_PROCS[proc].poll():
+                    dead.append(proc)
+
+            for d in dead:
+                LOG.with_field('pid', DAEMON_PROCS[d].pid).warning(
+                    '%s is dead, restarting' % proc)
+                del DAEMON_PROCS[d]
 
         except ChildProcessError:
             # We get this if there are no child processes
@@ -247,18 +241,29 @@ def main():
             _audit_daemons()
             Node.observe_this_node()
 
-        elif len(DAEMON_PIDS) == 0:
+        elif len(DAEMON_PROCS) == 0:
             n.state = Node.STATE_STOPPED
             return
 
         else:
             if running:
-                for pid in DAEMON_PIDS:
+                shutdown_commenced = time.time()
+                for proc in DAEMON_PROCS:
                     try:
-                        os.kill(pid, signal.SIGTERM)
+                        os.kill(DAEMON_PROCS[proc].pid, signal.SIGTERM)
                         LOG.info('Sent SIGTERM to %s (pid %s)'
-                                 % (DAEMON_PIDS.get(pid, 'unknown'), pid))
+                                 % (proc, DAEMON_PROCS[proc].pid))
                     except OSError as e:
-                        LOG.warn('Failed to send SIGTERM to %s: %s' % (pid, e))
+                        LOG.warn('Failed to send SIGTERM to %s: %s'
+                                 % (proc, e))
+
+            if time.time() - shutdown_commenced > 10:
+                LOG.warning('We have taken more than ten seconds to shut down')
+                for proc in DAEMON_PROCS:
+                    LOG.warning('%s daemon still running (pid %d)'
+                                % (proc, DAEMON_PROCS[proc].pid))
+                LOG.warning('Dumping thread traces')
+                emit_trace()
+                shutdown_commenced = time.time()
 
             running = False
