@@ -6,7 +6,6 @@ from shakenfist import blob
 from shakenfist.baseobject import DatabaseBackedObject as dbo
 from shakenfist.config import config
 from shakenfist.daemons import daemon
-from shakenfist import db
 from shakenfist import etcd
 from shakenfist import exceptions
 from shakenfist import images
@@ -24,7 +23,7 @@ from shakenfist.tasks import (QueueTask,
                               FloatNetworkInterfaceTask,
                               SnapshotTask,
                               FetchBlobTask)
-from shakenfist import net
+from shakenfist import network
 from shakenfist import networkinterface
 from shakenfist import scheduler
 from shakenfist.util import general as util_general
@@ -70,20 +69,11 @@ def handle(jobname, workitem):
 
             log_i.with_field('task_name', task.name()).info('Starting task')
 
-            # TODO(andy) Should network events also come through here eventually?
-            # Then this can be generalised to record events on networks/instances
-
-            # TODO(andy) This event should be recorded when it is recorded as
-            # dequeued in the DB. Currently it's reporting action on the item
-            # and calling it 'dequeue'.
-
-            if inst:
-                # TODO(andy) move to QueueTask
-                db.add_event('instance', inst.uuid, task.pretty_task_name(),
-                             'dequeued', None, 'Work item %s' % jobname)
-
             if isinstance(task, FetchImageTask):
-                image_fetch(task.url(), inst)
+                n = task.namespace()
+                if not n:
+                    n = Artifact.SHARED_WITH_ALL
+                image_fetch(task.url(), n, inst)
 
             elif isinstance(task, PreflightInstanceTask):
                 if (inst.state.value == dbo.STATE_DELETED or
@@ -109,12 +99,10 @@ def handle(jobname, workitem):
                     continue
 
                 instance_start(inst, task.network())
-                etcd.enqueue('%s-metrics' % config.NODE_NAME, {})
 
             elif isinstance(task, DeleteInstanceTask):
                 try:
                     instance_delete(inst)
-                    etcd.enqueue('%s-metrics' % config.NODE_NAME, {})
                 except Exception as e:
                     util_general.ignore_exception(
                         'instance %s delete task' % inst, e)
@@ -124,14 +112,13 @@ def handle(jobname, workitem):
                 etcd.enqueue('networknode', task)
 
             elif isinstance(task, SnapshotTask):
-                snapshot(inst, task.disk(),
-                         task.artifact_uuid(), task.blob_uuid())
+                snapshot(inst, task.disk(), task.artifact_uuid(), task.blob_uuid(),
+                         task.thin())
 
             elif isinstance(task, DeleteNetworkWhenClean):
                 # Check if any interfaces remain on network
-                task_network = net.Network.from_db(task.network_uuid())
-                ifaces = networkinterface.interfaces_for_network(task_network)
-                cur_interfaces = {i.uuid: i for i in ifaces}
+                task_network = network.Network.from_db(task.network_uuid())
+                cur_interfaces = task_network.networkinterfaces
 
                 if cur_interfaces:
                     LOG.with_network(task_network).error(
@@ -143,8 +130,8 @@ def handle(jobname, workitem):
                                          set(cur_interfaces))
                 if remain_interfaces:
                     # Queue task on a node with a remaining instance
-                    first_iface = cur_interfaces[remain_interfaces[0]]
-                    inst = instance.Instance.from_db(first_iface.instance_uuid)
+                    inst = instance.Instance.from_db(
+                        remain_interfaces[0].instance_uuid)
                     etcd.enqueue(inst.placement['node'],
                                  {'tasks': [
                                      DeleteNetworkWhenClean(task.network_uuid(),
@@ -158,7 +145,7 @@ def handle(jobname, workitem):
                                  DestroyNetworkTask(task.network_uuid()))
 
             elif isinstance(task, HypervisorDestroyNetworkTask):
-                n = net.Network.from_db(task.network_uuid())
+                n = network.Network.from_db(task.network_uuid())
                 n.delete_on_hypervisor()
 
             elif isinstance(task, FetchBlobTask):
@@ -223,48 +210,53 @@ def handle(jobname, workitem):
 
     finally:
         etcd.resolve(config.NODE_NAME, jobname)
-        if inst:
-            inst.add_event('tasks complete', 'dequeued',
-                           msg='Work item %s' % jobname)
-        log.info('Completed workitem')
 
 
-def image_fetch(url, inst):
-    a = Artifact.from_url(Artifact.TYPE_IMAGE, url)
+def image_fetch(url, namespace, inst):
+    a = Artifact.from_url(Artifact.TYPE_IMAGE, url, namespace=namespace,
+                          create_if_new=True)
     try:
         # TODO(andy): Wait up to 15 mins for another queue process to download
         # the required image. This will be changed to queue on a
         # "waiting_image_fetch" queue but this works now.
-        images.ImageFetchHelper(inst, url).get_image()
-        a.add_event('fetch', None, None, 'success')
-
-    except (exceptions.HTTPError, requests.exceptions.RequestException) as e:
-        LOG.with_field('image', url).info('Failed to fetch image')
+        images.ImageFetchHelper(inst, a).get_image()
+        a.add_event2('artifact fetch complete')
 
+    except (exceptions.HTTPError, requests.exceptions.RequestException,
+            requests.exceptions.ConnectionError) as e:
         # Clean common problems to store in events
         msg = str(e)
         if msg.find('Name or service not known'):
             msg = 'DNS error'
         if msg.find('No address associated with hostname'):
             msg = 'DNS error'
-        a.add_event('fetch', None, None, msg)
 
-        raise exceptions.ImageFetchTaskFailedException(
-            'Failed to fetch image: %s Exception: %s' % (url, e))
+        # If the artifact has never successfully downloaded, then we are
+        # clearly in an error state. However, if we already have a copy of the
+        # artifact and the serving web site is experiencing a transient error
+        # we should not mark the entire artifact as in error.
+        if (a.state.value in [Artifact.STATE_INITIAL, Artifact.STATE_CREATING] or
+                msg != 'DNS error'):
+            a.state = Artifact.STATE_ERROR
+            a.error = msg
+            raise exceptions.ImageFetchTaskFailedException(
+                'Failed to fetch image: %s Exception: %s' % (url, e))
+        else:
+            a.add_event2('Updating image failed, using already cached version',
+                         extra={'message': msg})
 
 
-def instance_preflight(inst, network):
+def instance_preflight(inst, netdescs):
     inst.state = instance.Instance.STATE_PREFLIGHT
 
     # Try to place on this node
     s = scheduler.Scheduler()
     try:
-        s.place_instance(inst, network, candidates=[config.NODE_NAME])
+        s.place_instance(inst, netdescs, candidates=[config.NODE_NAME])
         return None
 
     except exceptions.LowResourceException as e:
-        inst.add_event('schedule', 'retry', None,
-                       'insufficient resources: ' + str(e))
+        inst.add_event2('schedule failed, insufficient resources: ' + str(e))
 
     # Unsuccessful placement, check if reached placement attempt limit
     db_placement = inst.placement
@@ -283,36 +275,32 @@ def instance_preflight(inst, network):
                 if node != config.NODE_NAME:
                     candidates.append(node)
 
-        candidates = s.place_instance(inst, network,
-                                      candidates=candidates)
+        candidates = s.place_instance(inst, netdescs, candidates=candidates)
         inst.place_instance(candidates[0])
         return candidates[0]
 
     except exceptions.LowResourceException as e:
-        inst.add_event('schedule', 'failed', None,
-                       'insufficient resources: ' + str(e))
+        inst.add_event2('schedule failed, insufficient resources: ' + str(e))
         # This raise implies delete above
         raise exceptions.AbortInstanceStartException(
             'Unable to find suitable node')
 
 
-def instance_start(inst, network):
+def instance_start(inst, netdescs):
     if inst.state.value.endswith('-error'):
-        LOG.with_instance(inst).warning(
-            'You cannot start an instance in an error state.')
+        inst.add_event2('You cannot start an instance in an error state.')
         return
     if inst.state.value in (dbo.STATE_DELETE_WAIT, dbo.STATE_DELETED):
-        LOG.with_instance(inst).warning(
-            'You cannot start an instance which has been deleted.')
+        inst.add_event2('You cannot start an instance which has been deleted.')
         return
 
     with inst.get_lock(ttl=900, op='Instance start') as lock:
         try:
             # Ensure networks are connected to this node
             iface_uuids = []
-            for netdesc in network:
+            for netdesc in netdescs:
                 iface_uuids.append(netdesc['iface_uuid'])
-                n = net.Network.from_db(netdesc['network_uuid'])
+                n = network.Network.from_db(netdesc['network_uuid'])
                 if not n:
                     inst.enqueue_delete_due_error(
                         'missing network: %s' % netdesc['network_uuid'])
@@ -327,8 +315,18 @@ def instance_start(inst, network):
                 # detection code in the net daemon to work correctly.
                 ni = networkinterface.NetworkInterface.from_db(
                     netdesc['iface_uuid'])
-                ni.state = dbo.STATE_CREATED
+                if ni.state.value not in dbo.ACTIVE_STATES:
+                    inst.add_event2(
+                        'You cannot start an instance with an inactive network '
+                        'interface.', extra={
+                            'networkinterface': ni.uuid,
+                            'state': ni.state.value
+                        })
+                    inst.enqueue_delete_due_error(
+                        'Network interface is inactive')
+                    return
 
+                ni.state = dbo.STATE_CREATED
                 n.create_on_hypervisor()
                 n.ensure_mesh()
                 n.update_dhcp()
@@ -357,7 +355,6 @@ def instance_delete(inst):
         # in a transition state.
         if not inst.state.value.endswith('-error'):
             inst.state = dbo.STATE_DELETE_WAIT
-        db.add_event('instance', inst.uuid, 'queued', 'delete', None, None)
 
         # Create list of networks used by instance. We cannot use the
         # interfaces cached in the instance here, because the instance
@@ -375,9 +372,8 @@ def instance_delete(inst):
         inst.power_off()
 
         # Delete the instance's interfaces
-        with util_general.RecordedOperation('release network addresses', inst):
-            for ni in interfaces:
-                ni.delete()
+        for ni in interfaces:
+            ni.delete()
 
         # Create list of networks used by all other instances
         host_networks = []
@@ -392,39 +388,51 @@ def instance_delete(inst):
         inst.delete()
 
         # Check each network used by the deleted instance
-        for network in instance_networks:
-            n = net.Network.from_db(network)
+        for network_uuid in instance_networks:
+            n = network.Network.from_db(network_uuid)
             if n:
                 # If network used by another instance, only update
-                if network in host_networks:
+                if network_uuid in host_networks:
                     if n.state.value == dbo.STATE_DELETE_WAIT:
                         # Do not update a network about to be deleted
                         continue
-                    with util_general.RecordedOperation('deallocate ip address', inst):
-                        n.update_dhcp()
+                    n.update_dhcp()
                 else:
                     # Network not used by any other instance therefore delete
-                    with util_general.RecordedOperation('remove network from node', n):
-                        n.delete_on_hypervisor()
+                    n.delete_on_hypervisor()
 
 
-def snapshot(inst, disk, artifact_uuid, blob_uuid):
-    b = blob.snapshot_disk(disk, blob_uuid)
+def snapshot(inst, disk, artifact_uuid, blob_uuid, thin=False):
     a = Artifact.from_db(artifact_uuid)
+    if a.state.value == Artifact.STATE_DELETED:
+        # The artifact was deleted before the queued blob creation occurred
+        return
 
-    if b.state.value == blob.Blob.STATE_DELETED:
-        # The blob was deleted while it was being created
-        a.state = Artifact.STATE_ERROR
-    else:
+    try:
+        b = blob.snapshot_disk(disk, blob_uuid, thin=thin)
+    except exceptions.BlobDependencyMissing:
+        return
+
+    if a.state.value == Artifact.STATE_DELETED:
+        # The artifact was deleted while we were creating the blob, just delete
+        # the blob too.
+        b.state = blob.Blob.STATE_DELETED
+
+    try:
         b.ref_count_inc()
         a.state = Artifact.STATE_CREATED
+    except exceptions.BlobDeleted:
+        if a.state.value != Artifact.STATE_DELETED:
+            a.state = Artifact.STATE_ERROR
+    except exceptions.InvalidStateException:
+        b.ref_count_dec()
 
 
 class Monitor(daemon.WorkerPoolDaemon):
     def run(self):
         LOG.info('Starting')
 
-        while True:
+        while not self.exit.is_set():
             try:
                 self.reap_workers()
 
