@@ -60,7 +60,11 @@ class DHCP(object):
         t = self._read_template('dhcphosts.tmpl')
 
         instances = []
-        for ni in networkinterface.interfaces_for_network(self.network):
+        for ni_uuid in self.network.networkinterfaces:
+            ni = networkinterface.NetworkInterface.from_db(ni_uuid)
+            if not ni:
+                continue
+
             inst = instance.Instance.from_db(ni.instance_uuid)
             if not inst:
                 continue
@@ -90,6 +94,11 @@ class DHCP(object):
             if not psutil.pid_exists(pid):
                 return False
             os.kill(pid, sig)
+            if sig == signal.SIGKILL:
+                try:
+                    os.waitpid(pid, 0)
+                except ChildProcessError:
+                    pass
             return True
         return False
 
