@@ -104,8 +104,9 @@ api.add_resource(api_auth.AuthMetadatasEndpoint,
 api.add_resource(api_auth.AuthMetadataEndpoint,
                  '/auth/namespaces/<namespace>/metadata/<key>')
 
-api.add_resource(api_blob.BlobsEndpoint, '/blob')
-api.add_resource(api_blob.BlobEndpoint, '/blob/<blob_uuid>')
+api.add_resource(api_blob.BlobsEndpoint, '/blobs')
+api.add_resource(api_blob.BlobEndpoint, '/blobs/<blob_uuid>')
+api.add_resource(api_blob.BlobDataEndpoint, '/blobs/<blob_uuid>/data')
 
 api.add_resource(api_instance.InstancesEndpoint, '/instances')
 api.add_resource(api_instance.InstanceEndpoint, '/instances/<instance_ref>')
