@@ -1,5 +1,6 @@
 import flask
 from flask_jwt_extended import jwt_required, get_jwt_identity
+from functools import partial
 import json
 import os
 import requests
@@ -7,14 +8,14 @@ import shutil
 import time
 import uuid
 
-from shakenfist.artifact import Artifact, Artifacts, UPLOAD_URL
+from shakenfist.artifact import Artifact, Artifacts, UPLOAD_URL, namespace_filter
 from shakenfist.blob import Blob
 from shakenfist import baseobject
 from shakenfist import constants
 from shakenfist.daemons import daemon
+from shakenfist import eventlog
 from shakenfist.external_api import base as api_base
 from shakenfist.config import config
-from shakenfist import db
 from shakenfist import etcd
 from shakenfist import logutil
 from shakenfist.tasks import FetchImageTask
@@ -40,59 +41,68 @@ def arg_is_artifact_uuid(func):
     return wrapper
 
 
+def requires_artifact_ownership(func):
+    # Requires that @arg_is_artifact_uuid has already run
+    def wrapper(*args, **kwargs):
+        if not kwargs.get('artifact_from_db'):
+            LOG.with_field('artifact', kwargs['artifact_uuid']).info(
+                'Artifact not found, kwarg missing')
+            return api_base.error(404, 'artifact not found')
+
+        a = kwargs['artifact_from_db']
+        if get_jwt_identity()[0] not in [a.namespace, 'system']:
+            LOG.with_object(a).info(
+                'Artifact not found, ownership test in decorator')
+            return api_base.error(404, 'artifact not found')
+
+        return func(*args, **kwargs)
+    return wrapper
+
+
+def requires_artifact_access(func):
+    # Requires that @arg_is_artifact_uuid has already run
+    def wrapper(*args, **kwargs):
+        if not kwargs.get('artifact_from_db'):
+            LOG.with_field('artifact', kwargs['artifact_uuid']).info(
+                'Artifact not found, kwarg missing')
+            return api_base.error(404, 'artifact not found')
+
+        a = kwargs['artifact_from_db']
+        if (a.namespace != Artifact.SHARED_WITH_ALL and
+                get_jwt_identity()[0] not in [a.namespace, 'system']):
+            LOG.with_object(a).info(
+                'Artifact not found, ownership test in decorator')
+            return api_base.error(404, 'artifact not found')
+
+        return func(*args, **kwargs)
+    return wrapper
+
+
 class ArtifactEndpoint(api_base.Resource):
-    @jwt_required
+    @jwt_required()
     @arg_is_artifact_uuid
+    @requires_artifact_access
     def get(self, artifact_uuid=None, artifact_from_db=None):
         with etcd.ThreadLocalReadOnlyCache():
             return artifact_from_db.external_view()
 
-    @jwt_required
+    @jwt_required()
     @arg_is_artifact_uuid
+    @requires_artifact_ownership
     def delete(self, artifact_uuid=None, artifact_from_db=None):
-        """Delete an artifact from the cluster
-
-        Artifacts can only be deleted from the system if they are not in use.
-        The actual deletion of the on-disk files is left to the cleaner daemon.
-
-        It is acknowledged that there is a potential race condition between the
-        check that an artifact is not in use and the marking of the artifact as
-        deleted. This is only caused by a user simultaneously deleting an
-        artifact and attempting to start a VM using it. It is recommended that
-        the user does not do that.
-        """
-        # TODO(andy): Enforce namespace permissions when snapshots have namespaces
-        # TODO(mikal): this should all be refactored to be in the object
-
         if artifact_from_db.state.value == Artifact.STATE_DELETED:
-            # Already deleted, nothing to do.
             return
-
-        # Check for instances using a blob referenced by the artifact.
-        blobs = []
-        sole_ref_in_use = []
-        for blob_index in artifact_from_db.get_all_indexes():
-            b = Blob.from_db(blob_index['blob_uuid'])
-            if b:
-                blobs.append(b)
-                if b.ref_count == 1:
-                    sole_ref_in_use += b.instances
-        if sole_ref_in_use:
-            return api_base.error(
-                400, 'Cannot delete last reference to blob in use by instance (%s)' % (
-                    ', '.join(sole_ref_in_use), ))
-
         artifact_from_db.delete()
-        for b in blobs:
-            b.ref_count_dec()
+        return artifact_from_db.external_view()
 
 
 class ArtifactsEndpoint(api_base.Resource):
-    @jwt_required
+    @jwt_required()
     def get(self, node=None):
         retval = []
         with etcd.ThreadLocalReadOnlyCache():
-            for a in Artifacts(filters=[baseobject.active_states_filter]):
+            for a in Artifacts(filters=[baseobject.active_states_filter,
+                                        partial(namespace_filter, get_jwt_identity()[0])]):
                 if node:
                     idx = a.most_recent_index
                     if 'blob_uuid' in idx:
@@ -103,23 +113,67 @@ class ArtifactsEndpoint(api_base.Resource):
                     retval.append(a.external_view())
         return retval
 
-    @jwt_required
-    def post(self, url=None):
+    @jwt_required()
+    @api_base.requires_namespace_exist
+    def post(self, url=None, shared=False, namespace=None):
         # The only artifact type you can force the cluster to fetch is an
         # image, so TYPE_IMAGE is assumed here. We ensure that the image exists
         # in the database in an initial state here so that it will show up in
         # image list requests. The image is fetched by the queued job later.
-        a = Artifact.from_url(Artifact.TYPE_IMAGE, url)
+        if not namespace:
+            namespace = get_jwt_identity()[0]
+
+        # If accessing a foreign namespace, we need to be an admin
+        if get_jwt_identity()[0] not in [namespace, 'system']:
+            return api_base.error(404, 'namespace not found')
+
+        # Only admin can create shared artifacts
+        if shared:
+            if get_jwt_identity()[0] != 'system':
+                return api_base.error(
+                    403, 'only the system namespace can create shared artifacts')
+            namespace = Artifact.SHARED_WITH_ALL
+        a = Artifact.from_url(Artifact.TYPE_IMAGE, url, namespace=namespace,
+                              create_if_new=True)
 
         etcd.enqueue(config.NODE_NAME, {
-            'tasks': [FetchImageTask(url)],
+            'tasks': [FetchImageTask(url, namespace=namespace)],
         })
         return a.external_view()
 
+    @jwt_required()
+    @api_base.requires_namespace_exist
+    def delete(self, confirm=False, namespace=None):
+        """Delete all artifacts in the namespace."""
+
+        if confirm is not True:
+            return api_base.error(400, 'parameter confirm is not set true')
+
+        if get_jwt_identity()[0] == 'system':
+            if not isinstance(namespace, str):
+                # A client using a system key must specify the namespace. This
+                # ensures that deleting all artifacts in the cluster (by
+                # specifying namespace='system') is a deliberate act.
+                return api_base.error(400, 'system user must specify parameter namespace')
+
+        else:
+            if namespace and namespace != get_jwt_identity()[0]:
+                return api_base.error(401, 'you cannot delete other namespaces')
+            namespace = get_jwt_identity()[0]
+
+        deleted = []
+        for a in Artifacts([partial(namespace_filter, namespace)]):
+            a.delete()
+            deleted.append(a.uuid)
+
+        return deleted
+
 
 class ArtifactUploadEndpoint(api_base.Resource):
-    @jwt_required
-    def post(self, artifact_name=None, upload_uuid=None, source_url=None):
+    @jwt_required()
+    @api_base.requires_namespace_exist
+    def post(self, artifact_name=None, upload_uuid=None, source_url=None,
+             shared=False, namespace=None):
         u = Upload.from_db(upload_uuid)
         if not u:
             return api_base.error(404, 'upload not found')
@@ -146,7 +200,23 @@ class ArtifactUploadEndpoint(api_base.Resource):
         if not source_url:
             source_url = ('%s%s/%s'
                           % (UPLOAD_URL, get_jwt_identity()[0], artifact_name))
-        a = Artifact.from_url(Artifact.TYPE_IMAGE, source_url)
+
+        if not namespace:
+            namespace = get_jwt_identity()[0]
+
+        # If accessing a foreign namespace, we need to be an admin
+        if get_jwt_identity()[0] not in [namespace, 'system']:
+            return api_base.error(404, 'namespace not found')
+
+        # Only admin can create shared artifacts
+        if shared:
+            if get_jwt_identity()[0] != 'system':
+                return api_base.error(
+                    403, 'only the system namespace can create shared artifacts')
+            namespace = Artifact.SHARED_WITH_ALL
+
+        a = Artifact.from_url(Artifact.TYPE_IMAGE, source_url,
+                              namespace=namespace, create_if_new=True)
 
         with a.get_lock(ttl=(12 * constants.LOCK_REFRESH_SECONDS),
                         timeout=config.MAX_IMAGE_TRANSFER_SECONDS):
@@ -170,23 +240,28 @@ class ArtifactUploadEndpoint(api_base.Resource):
             b.observe()
             b.request_replication()
 
-            a.state = Artifact.STATE_CREATED
-            a.add_event('upload', None, None, 'success')
+            a.add_event2('upload complete')
             a.add_index(b.uuid)
             a.state = Artifact.STATE_CREATED
+
+            u.hard_delete()
             return a.external_view()
 
 
 class ArtifactEventsEndpoint(api_base.Resource):
-    @jwt_required
-    # TODO(andy): Should images be owned? Personalised images should be owned.
-    def get(self, artifact_uuid):
-        return list(db.get_events('artifact', artifact_uuid))
+    @jwt_required()
+    @arg_is_artifact_uuid
+    @requires_artifact_access
+    @api_base.redirect_to_eventlog_node
+    def get(self, artifact_uuid=None, artifact_from_db=None):
+        with eventlog.EventLog('artifact', artifact_uuid) as eventdb:
+            return list(eventdb.read_events())
 
 
 class ArtifactVersionsEndpoint(api_base.Resource):
-    @jwt_required
+    @jwt_required()
     @arg_is_artifact_uuid
+    @requires_artifact_access
     def get(self, artifact_uuid=None, artifact_from_db=None):
         retval = []
         for idx in artifact_from_db.get_all_indexes():
@@ -196,8 +271,9 @@ class ArtifactVersionsEndpoint(api_base.Resource):
             retval.append(bout)
         return retval
 
-    @jwt_required
+    @jwt_required()
     @arg_is_artifact_uuid
+    @requires_artifact_ownership
     def post(self, artifact_uuid=None, artifact_from_db=None,
              max_versions=config.ARTIFACT_MAX_VERSIONS_DEFAULT):
         try:
@@ -208,8 +284,9 @@ class ArtifactVersionsEndpoint(api_base.Resource):
 
 
 class ArtifactVersionEndpoint(api_base.Resource):
-    @jwt_required
+    @jwt_required()
     @arg_is_artifact_uuid
+    @requires_artifact_ownership
     def delete(self, artifact_uuid=None, artifact_from_db=None, version_id=0):
         try:
             ver_index = int(version_id)
@@ -222,6 +299,6 @@ class ArtifactVersionEndpoint(api_base.Resource):
                 artifact_from_db.del_index(idx['index'])
                 if len(indexes) == 1:
                     artifact_from_db.state = Artifact.STATE_DELETED
-                return
+                return artifact_from_db.external_view()
 
         return api_base.error(404, 'artifact index not found')
