@@ -13,6 +13,7 @@ from shakenfist.config import config
 from shakenfist.daemons import daemon
 from shakenfist import db
 from shakenfist import etcd
+from shakenfist import eventlog
 from shakenfist import exceptions
 from shakenfist.external_api import (
     base as api_base,
@@ -20,7 +21,8 @@ from shakenfist.external_api import (
 from shakenfist import instance
 from shakenfist.ipmanager import IPManager
 from shakenfist import logutil
-from shakenfist import net
+from shakenfist import network as sfnet  # Unfortunate, but we have an API arg
+# called network too.
 from shakenfist.networkinterface import NetworkInterface
 from shakenfist.node import Node
 from shakenfist import scheduler
@@ -42,13 +44,13 @@ SCHEDULER = None
 
 
 class InstanceEndpoint(api_base.Resource):
-    @jwt_required
+    @jwt_required()
     @api_base.arg_is_instance_ref
     @api_base.requires_instance_ownership
     def get(self, instance_ref=None, instance_from_db=None):
         return instance_from_db.external_view()
 
-    @jwt_required
+    @jwt_required()
     @api_base.arg_is_instance_ref
     @api_base.requires_instance_ownership
     @api_base.requires_namespace_exist
@@ -72,7 +74,7 @@ class InstanceEndpoint(api_base.Resource):
         instance_from_db.enqueue_delete_remote(node)
 
         # Return UUID in case API call was made using object name
-        return {'uuid': instance_from_db.uuid}
+        return instance_from_db.external_view()
 
 
 def _artifact_safety_checks(a):
@@ -81,11 +83,14 @@ def _artifact_safety_checks(a):
     if a.state.value != Artifact.STATE_CREATED:
         return api_base.error(
             404, 'artifact not ready (state=%s)' % a.state.value)
+    if (get_jwt_identity()[0] != 'system' and
+            a.namespace not in [get_jwt_identity()[0], Artifact.SHARED_WITH_ALL]):
+        return api_base.error(404, 'artifact not found')
     return
 
 
 class InstancesEndpoint(api_base.Resource):
-    @jwt_required
+    @jwt_required()
     def get(self, all=False):
         with etcd.ThreadLocalReadOnlyCache():
             filters = [partial(baseobject.namespace_filter,
@@ -99,12 +104,12 @@ class InstancesEndpoint(api_base.Resource):
                 retval.append(i.external_view())
             return retval
 
-    @jwt_required
+    @jwt_required()
     @api_base.requires_namespace_exist
     def post(self, name=None, cpus=None, memory=None, network=None, disk=None,
              ssh_key=None, user_data=None, placed_on=None, namespace=None,
              video=None, uefi=False, configdrive=None, metadata=None,
-             nvram_template=None, secure_boot=False):
+             nvram_template=None, secure_boot=False, side_channels=None):
         global SCHEDULER
 
         instance_uuid = str(uuid.uuid4())
@@ -170,7 +175,9 @@ class InstancesEndpoint(api_base.Resource):
             elif disk_base.startswith('label:'):
                 label = disk_base[len('label:'):]
                 a = Artifact.from_url(
-                    Artifact.TYPE_LABEL, '%s%s/%s' % (LABEL_URL, get_jwt_identity()[0], label))
+                    Artifact.TYPE_LABEL,
+                    '%s%s/%s' % (LABEL_URL, get_jwt_identity()[0], label),
+                    namespace=namespace)
                 err = _artifact_safety_checks(a)
                 if err:
                     return err
@@ -193,9 +200,11 @@ class InstancesEndpoint(api_base.Resource):
 
             elif disk_base.startswith(UPLOAD_URL) or disk_base.startswith(LABEL_URL):
                 if disk_base.startswith(UPLOAD_URL):
-                    a = Artifact.from_url(Artifact.TYPE_IMAGE, disk_base)
+                    a = Artifact.from_url(Artifact.TYPE_IMAGE, disk_base,
+                                          namespace=namespace)
                 else:
-                    a = Artifact.from_url(Artifact.TYPE_LABEL, disk_base)
+                    a = Artifact.from_url(Artifact.TYPE_LABEL, disk_base,
+                                          namespace=namespace)
                 err = _artifact_safety_checks(a)
                 if err:
                     return err
@@ -212,7 +221,8 @@ class InstancesEndpoint(api_base.Resource):
                 # We ensure that the image exists in the database in an initial state
                 # here so that it will show up in image list requests. The image is
                 # fetched by the queued job later.
-                Artifact.from_url(Artifact.TYPE_IMAGE, disk_base)
+                Artifact.from_url(Artifact.TYPE_IMAGE, disk_base,
+                                  namespace=namespace, create_if_new=True)
 
             transformed_disk.append(d)
 
@@ -225,7 +235,8 @@ class InstancesEndpoint(api_base.Resource):
             if nvram_template.startswith('label:'):
                 label = nvram_template[len('label:'):]
                 url = '%s%s/%s' % (LABEL_URL, get_jwt_identity()[0], label)
-                a = Artifact.from_url(Artifact.TYPE_LABEL, url)
+                a = Artifact.from_url(
+                    Artifact.TYPE_LABEL, url, namespace=namespace)
                 err = _artifact_safety_checks(a)
                 if err:
                     return err
@@ -271,7 +282,7 @@ class InstancesEndpoint(api_base.Resource):
                 # Allow network to be specified by name or UUID (and error early
                 # if not found)
                 try:
-                    n = net.Network.from_db_by_ref(netdesc['network_uuid'])
+                    n = sfnet.Network.from_db_by_ref(netdesc['network_uuid'])
                 except exceptions.MultipleObjects as e:
                     return api_base.error(400, str(e))
 
@@ -289,7 +300,7 @@ class InstancesEndpoint(api_base.Resource):
                                               'network specification requests an address outside the '
                                               'range of the network')
 
-                if n.state.value != net.Network.STATE_CREATED:
+                if n.state.value != sfnet.Network.STATE_CREATED:
                     return api_base.error(406, 'network %s is not ready (%s)' % (n.uuid, n.state.value))
                 if n.namespace != namespace:
                     return api_base.error(404, 'network %s does not exist' % n.uuid)
@@ -327,7 +338,8 @@ class InstancesEndpoint(api_base.Resource):
             requested_placement=placed_on,
             nvram_template=nvram_template,
             secure_boot=secure_boot,
-            machine_type=machine_type
+            machine_type=machine_type,
+            side_channels=side_channels
         )
 
         # Initialise metadata
@@ -342,7 +354,7 @@ class InstancesEndpoint(api_base.Resource):
         updated_networks = []
         if network:
             for netdesc in network:
-                n = net.Network.from_db(netdesc['network_uuid'])
+                n = sfnet.Network.from_db(netdesc['network_uuid'])
                 if not n:
                     m = 'missing network %s during IP allocation phase' % (
                         netdesc['network_uuid'])
@@ -354,26 +366,31 @@ class InstancesEndpoint(api_base.Resource):
                 # (thanks OpenStack Kolla), which are special cased here. To not
                 # have an address, you use a detailed netdesc and specify
                 # address=none.
-                if 'address' in netdesc and util_general.noneish(netdesc['address']):
-                    netdesc['address'] = None
-                else:
-                    with db.get_lock('ipmanager', None,  netdesc['network_uuid'],
-                                     ttl=120, op='Network allocate IP'):
-                        db.add_event('network', netdesc['network_uuid'], 'allocate address',
-                                     None, None, inst.uuid)
-                        ipm = IPManager.from_db(netdesc['network_uuid'])
-                        if 'address' not in netdesc or not netdesc['address']:
-                            netdesc['address'] = ipm.get_random_free_address(
-                                inst.unique_label())
-                        else:
-                            if not ipm.reserve(netdesc['address'], inst.unique_label()):
-                                m = 'failed to reserve an IP on network %s' % (
-                                    netdesc['network_uuid'])
-                                inst.enqueue_delete_due_error(m)
-                                return api_base.error(409, 'address %s in use' %
-                                                      netdesc['address'])
+                try:
+                    if 'address' in netdesc and util_general.noneish(netdesc['address']):
+                        netdesc['address'] = None
+                    else:
+                        with db.get_lock('ipmanager', None,  netdesc['network_uuid'],
+                                         ttl=120, op='Network allocate IP'):
+                            ipm = IPManager.from_db(netdesc['network_uuid'])
+                            if 'address' not in netdesc or not netdesc['address']:
+                                netdesc['address'] = ipm.get_random_free_address(
+                                    inst.unique_label())
+                                inst.add_event2(
+                                    'allocated ip address', extra=netdesc)
+                            else:
+                                if not ipm.reserve(netdesc['address'], inst.unique_label()):
+                                    m = 'failed to reserve an IP on network %s' % (
+                                        netdesc['network_uuid'])
+                                    inst.enqueue_delete_due_error(m)
+                                    return api_base.error(409, 'address %s in use' %
+                                                          netdesc['address'])
 
-                        ipm.persist()
+                            ipm.persist()
+                except exceptions.CongestedNetwork as e:
+                    inst.enqueue_delete_due_error(
+                        'cannot allocate address: %s' % e)
+                    return api_base.error(507, str(e))
 
                 if 'model' not in netdesc or not netdesc['model']:
                     netdesc['model'] = 'virtio'
@@ -386,15 +403,20 @@ class InstancesEndpoint(api_base.Resource):
                     iface_uuid, netdesc, inst.uuid, order)
                 order += 1
 
-                if 'float' in netdesc and netdesc['float']:
-                    err = api_util.assign_floating_ip(ni)
-                    if err:
-                        inst.enqueue_delete_due_error(
-                            'interface float failed: %s' % err)
-                        return err
+                try:
+                    if 'float' in netdesc and netdesc['float']:
+                        err = api_util.assign_floating_ip(ni)
+                        if err:
+                            inst.enqueue_delete_due_error(
+                                'interface float failed: %s' % err)
+                            return err
 
-                    float_tasks.append(FloatNetworkInterfaceTask(
-                        netdesc['network_uuid'], iface_uuid))
+                        float_tasks.append(FloatNetworkInterfaceTask(
+                            netdesc['network_uuid'], iface_uuid))
+                except exceptions.CongestedNetwork as e:
+                    inst.enqueue_delete_due_error(
+                        'cannot allocate address: %s' % e)
+                    return api_base.error(507, str(e))
 
                 # Include the interface uuid in the network description we
                 # pass through to the instance start task.
@@ -419,14 +441,13 @@ class InstancesEndpoint(api_base.Resource):
                 placement = placed_on
 
         except exceptions.LowResourceException as e:
-            inst.add_event('schedule', 'failed', None,
-                           'Insufficient resources: ' + str(e))
+            inst.add_event2(
+                'schedule failed, insufficient resources: %s' % str(e))
             inst.enqueue_delete_due_error('scheduling failed')
             return api_base.error(507, str(e), suppress_traceback=True)
 
         except exceptions.CandidateNodeNotFoundException as e:
-            inst.add_event('schedule', 'failed', None,
-                           'Candidate node not found: ' + str(e))
+            inst.add_event2('schedule failed, node not found: %s' % str(e))
             inst.enqueue_delete_due_error('scheduling failed')
             return api_base.error(404, 'node not found: %s' % e, suppress_traceback=True)
 
@@ -439,9 +460,12 @@ class InstancesEndpoint(api_base.Resource):
             disk_base = disk.get('base')
             if disk.get('blob_uuid'):
                 tasks.append(FetchImageTask(
-                    '%s%s' % (BLOB_URL, disk['blob_uuid']), inst.uuid))
+                    '%s%s' % (BLOB_URL, disk['blob_uuid']),
+                    namespace=namespace, instance_uuid=inst.uuid))
             elif not util_general.noneish(disk_base):
-                tasks.append(FetchImageTask(disk['base'], inst.uuid))
+                tasks.append(FetchImageTask(
+                    disk['base'],
+                    namespace=namespace, instance_uuid=inst.uuid))
         tasks.append(StartInstanceTask(inst.uuid, network))
         tasks.extend(float_tasks)
 
@@ -449,7 +473,7 @@ class InstancesEndpoint(api_base.Resource):
         etcd.enqueue(placement, {'tasks': tasks})
         return inst.external_view()
 
-    @jwt_required
+    @jwt_required()
     @api_base.requires_namespace_exist
     def delete(self, confirm=False, namespace=None):
         """Delete all instances in the namespace."""
@@ -471,8 +495,7 @@ class InstancesEndpoint(api_base.Resource):
 
         waiting_for = []
         tasks_by_node = defaultdict(list)
-        for inst in instance.Instances([partial(baseobject.namespace_filter, namespace),
-                                        instance.active_states_filter]):
+        for inst in instance.Instances([partial(baseobject.namespace_filter, namespace)]):
             # If this instance is not on a node, just do the DB cleanup locally
             db_placement = inst.placement
             if not db_placement.get('node'):
@@ -490,7 +513,7 @@ class InstancesEndpoint(api_base.Resource):
 
 
 class InstanceInterfacesEndpoint(api_base.Resource):
-    @jwt_required
+    @jwt_required()
     @api_base.arg_is_instance_ref
     @api_base.requires_instance_ownership
     def get(self, instance_ref=None, instance_from_db=None):
@@ -504,87 +527,101 @@ class InstanceInterfacesEndpoint(api_base.Resource):
 
 
 class InstanceEventsEndpoint(api_base.Resource):
-    @jwt_required
+    @jwt_required()
     @api_base.arg_is_instance_ref
     @api_base.requires_instance_ownership
+    @api_base.redirect_to_eventlog_node
     def get(self, instance_ref=None, instance_from_db=None):
-        return list(db.get_events('instance', instance_from_db.uuid))
+        with eventlog.EventLog('instance', instance_from_db.uuid) as eventdb:
+            return list(eventdb.read_events())
 
 
 class InstanceRebootSoftEndpoint(api_base.Resource):
-    @jwt_required
+    @jwt_required()
     @api_base.arg_is_instance_ref
     @api_base.requires_instance_ownership
     @api_base.redirect_instance_request
     @api_base.requires_instance_active
     def post(self, instance_ref=None, instance_from_db=None):
-        with instance_from_db.get_lock(op='Instance reboot soft'):
-            instance_from_db.add_event('api', 'soft reboot')
-            return instance_from_db.reboot(hard=False)
+        try:
+            with instance_from_db.get_lock(op='Instance reboot soft'):
+                return instance_from_db.reboot(hard=False)
+        except exceptions.InvalidLifecycleState as e:
+            return api_base.error(409, e)
 
 
 class InstanceRebootHardEndpoint(api_base.Resource):
-    @jwt_required
+    @jwt_required()
     @api_base.arg_is_instance_ref
     @api_base.requires_instance_ownership
     @api_base.redirect_instance_request
     @api_base.requires_instance_active
     def post(self, instance_ref=None, instance_from_db=None):
-        with instance_from_db.get_lock(op='Instance reboot hard'):
-            instance_from_db.add_event('api', 'hard reboot')
-            return instance_from_db.reboot(hard=True)
+        try:
+            with instance_from_db.get_lock(op='Instance reboot hard'):
+                return instance_from_db.reboot(hard=True)
+        except exceptions.InvalidLifecycleState as e:
+            return api_base.error(409, e)
 
 
 class InstancePowerOffEndpoint(api_base.Resource):
-    @jwt_required
+    @jwt_required()
     @api_base.arg_is_instance_ref
     @api_base.requires_instance_ownership
     @api_base.redirect_instance_request
     @api_base.requires_instance_active
     def post(self, instance_ref=None, instance_from_db=None):
-        with instance_from_db.get_lock(op='Instance power off'):
-            instance_from_db.add_event('api', 'poweroff')
-            return instance_from_db.power_off()
+        try:
+            with instance_from_db.get_lock(op='Instance power off'):
+                return instance_from_db.power_off()
+        except exceptions.InvalidLifecycleState as e:
+            return api_base.error(409, e)
 
 
 class InstancePowerOnEndpoint(api_base.Resource):
-    @jwt_required
+    @jwt_required()
     @api_base.arg_is_instance_ref
     @api_base.requires_instance_ownership
     @api_base.redirect_instance_request
     @api_base.requires_instance_active
     def post(self, instance_ref=None, instance_from_db=None):
-        with instance_from_db.get_lock(op='Instance power on'):
-            instance_from_db.add_event('api', 'poweron')
-            return instance_from_db.power_on()
+        try:
+            with instance_from_db.get_lock(op='Instance power on'):
+                return instance_from_db.power_on()
+        except exceptions.InvalidLifecycleState as e:
+            return api_base.error(409, e)
 
 
 class InstancePauseEndpoint(api_base.Resource):
-    @jwt_required
+    @jwt_required()
     @api_base.arg_is_instance_ref
     @api_base.requires_instance_ownership
     @api_base.redirect_instance_request
     @api_base.requires_instance_active
     def post(self, instance_ref=None, instance_from_db=None):
-        with instance_from_db.get_lock(op='Instance pause'):
-            instance_from_db.add_event('api', 'pause')
-            return instance_from_db.pause()
+        try:
+            with instance_from_db.get_lock(op='Instance pause'):
+                return instance_from_db.pause()
+        except exceptions.InvalidLifecycleState as e:
+            return api_base.error(409, e)
 
 
 class InstanceUnpauseEndpoint(api_base.Resource):
-    @jwt_required
+    @jwt_required()
     @api_base.arg_is_instance_ref
     @api_base.requires_instance_ownership
     @api_base.redirect_instance_request
     @api_base.requires_instance_active
     def post(self, instance_ref=None, instance_from_db=None):
-        with instance_from_db.get_lock(op='Instance unpause'):
-            instance_from_db.add_event('api', 'unpause')
-            return instance_from_db.unpause()
+        try:
+            with instance_from_db.get_lock(op='Instance unpause'):
+                return instance_from_db.unpause()
+        except exceptions.InvalidLifecycleState as e:
+            return api_base.error(409, e)
 
 
 class InstanceMetadatasEndpoint(api_base.Resource):
-    @jwt_required
+    @jwt_required()
     @api_base.arg_is_instance_ref
     @api_base.requires_instance_ownership
     def get(self, instance_ref=None, instance_from_db=None):
@@ -593,7 +630,7 @@ class InstanceMetadatasEndpoint(api_base.Resource):
             return {}
         return md
 
-    @jwt_required
+    @jwt_required()
     @api_base.arg_is_instance_ref
     @api_base.requires_instance_ownership
     def post(self, instance_ref=None, key=None, value=None, instance_from_db=None):
@@ -633,7 +670,7 @@ def _validate_instance_metadata(key, value):
 
 
 class InstanceMetadataEndpoint(api_base.Resource):
-    @jwt_required
+    @jwt_required()
     @api_base.arg_is_instance_ref
     @api_base.requires_instance_ownership
     def put(self, instance_ref=None, key=None, value=None, instance_from_db=None):
@@ -643,7 +680,7 @@ class InstanceMetadataEndpoint(api_base.Resource):
         return api_util.metadata_putpost(
             'instance', instance_from_db.uuid, key, value)
 
-    @jwt_required
+    @jwt_required()
     @api_base.arg_is_instance_ref
     @api_base.requires_instance_ownership
     def delete(self, instance_ref=None, key=None, instance_from_db=None):
@@ -659,7 +696,7 @@ class InstanceMetadataEndpoint(api_base.Resource):
 
 
 class InstanceConsoleDataEndpoint(api_base.Resource):
-    @jwt_required
+    @jwt_required()
     @api_base.arg_is_instance_ref
     @api_base.requires_instance_ownership
     @api_base.redirect_instance_request
@@ -681,11 +718,11 @@ class InstanceConsoleDataEndpoint(api_base.Resource):
 
         resp = flask.Response(
             instance_from_db.get_console_data(parsed_length),
-            mimetype='text/plain')
+            mimetype='applicaton/octet-stream')
         resp.status_code = 200
         return resp
 
-    @jwt_required
+    @jwt_required()
     @api_base.arg_is_instance_ref
     @api_base.requires_instance_ownership
     @api_base.redirect_instance_request
