@@ -61,7 +61,6 @@ class Node(dbo):
         })
         n = Node.from_db(name)
         n.state = cls.STATE_CREATED
-        n.add_event('db record creation', None)
         return n
 
     @classmethod
@@ -85,7 +84,7 @@ class Node(dbo):
             'ip': self.ip,
             'state': self.state.value,
             'lastseen': self.last_seen,
-            'release': self.installed_version
+            'version': self.installed_version
         }
 
     # Static values
@@ -106,6 +105,20 @@ class Node(dbo):
     def installed_version(self):
         return self._db_get_attribute('observed').get('release')
 
+    @property
+    def blobs(self):
+        return self._db_get_attribute('blobs').get('blobs', [])
+
+    @blobs.setter
+    def blobs(self, value):
+        self._db_set_attribute('blobs', {'blobs': value})
+
+    def add_blob(self, blob):
+        self._add_item_in_attribute_list('blobs', blob)
+
+    def remove_blob(self, blob):
+        self._remove_item_in_attribute_list('blobs', blob)
+
 
 class Nodes(dbo_iter):
     def __iter__(self):
