@@ -46,6 +46,7 @@ class InstanceTask(QueueTask):
     def __init__(self, instance_uuid, network=None):
         super(InstanceTask, self).__init__()
         self._instance_uuid = instance_uuid
+
         # Only set network if deliberately set in function paramater. This
         # avoids setting _network to None which is not iterable.
         self._network = []
@@ -84,11 +85,10 @@ class StartInstanceTask(InstanceTask):
 class DeleteInstanceTask(InstanceTask):
     _name = 'instance_delete'
 
+
 #
 # Network Tasks
 #
-
-
 class NetworkTask(QueueTask):
     def __init__(self, network_uuid):
         super(NetworkTask, self).__init__()
@@ -113,6 +113,14 @@ class DeployNetworkTask(NetworkTask):
     _name = 'network_deploy'
 
 
+class DestroyNetworkTask(NetworkTask):
+    _name = 'network_destroy'
+
+
+class HypervisorDestroyNetworkTask(NetworkTask):
+    _name = 'hypervisor_network_destroy'
+
+
 class UpdateDHCPNetworkTask(NetworkTask):
     _name = 'network_update_dhcp'
 
@@ -125,6 +133,21 @@ class RemoveNATNetworkTask(NetworkTask):
     _name = 'network_remove_nat'
 
 
+class DeleteNetworkWhenClean(NetworkTask):
+    _name = 'network_delete_when_clean'
+
+    def __init__(self, network_uuid, wait_interfaces):
+        super(DeleteNetworkWhenClean, self).__init__(network_uuid)
+        self._wait_interfaces = wait_interfaces
+
+    def wait_interfaces(self):
+        return self._wait_interfaces
+
+    def obj_dict(self):
+        return {**super(DeleteNetworkWhenClean, self).obj_dict(),
+                'wait_interfaces': self._wait_interfaces}
+
+
 #
 # NetworkInterface Tasks
 #
@@ -173,27 +196,32 @@ class DefloatNetworkInterfaceTask(NetworkInterfaceTask):
 
 
 class ImageTask(QueueTask):
-    def __init__(self, url):
+    def __init__(self, url, namespace=None):
         super(ImageTask, self).__init__()
         self._url = url
+        self._namespace = namespace
 
         if not isinstance(url, str):
             raise NoURLImageFetchTaskException
 
     def obj_dict(self):
         return {**super(ImageTask, self).obj_dict(),
-                'url': self._url}
+                'url': self._url,
+                'namespace': self._namespace}
 
     # Data methods
     def url(self):
         return self._url
 
+    def namespace(self):
+        return self._namespace
+
 
 class FetchImageTask(ImageTask):
     _name = 'image_fetch'
 
-    def __init__(self, url, instance_uuid=None):
-        super(FetchImageTask, self).__init__(url)
+    def __init__(self, url, namespace=None, instance_uuid=None):
+        super(FetchImageTask, self).__init__(url, namespace=namespace)
         self._instance_uuid = instance_uuid
 
     def obj_dict(self):
@@ -203,3 +231,69 @@ class FetchImageTask(ImageTask):
     # Data methods
     def instance_uuid(self):
         return self._instance_uuid
+
+#
+# Snapshot Tasks
+#
+
+
+class SnapshotTask(QueueTask):
+    _name = 'snapshot'
+
+    def __init__(self, instance_uuid, disk, artifact_uuid, blob_uuid, thin=False):
+        super(SnapshotTask, self).__init__()
+        self._instance_uuid = instance_uuid
+        self._disk = disk
+        self._artifact_uuid = artifact_uuid
+        self._blob_uuid = blob_uuid
+        self._thin = thin
+
+    def obj_dict(self):
+        return {
+            **super(SnapshotTask, self).obj_dict(),
+            'instance_uuid': self._instance_uuid,
+            'disk': self._disk,
+            'artifact_uuid': self._artifact_uuid,
+            'blob_uuid': self._blob_uuid,
+            'thin': self._thin
+        }
+
+    # Data methods
+    def instance_uuid(self):
+        return self._instance_uuid
+
+    def disk(self):
+        return self._disk
+
+    def artifact_uuid(self):
+        return self._artifact_uuid
+
+    def blob_uuid(self):
+        return self._blob_uuid
+
+    def thin(self):
+        return self._thin
+
+#
+# Blob tasks
+#
+
+
+class BlobTask(QueueTask):
+    def __init__(self, blob_uuid):
+        super(BlobTask, self).__init__()
+        self._blob_uuid = blob_uuid
+
+    def obj_dict(self):
+        return {
+            **super(BlobTask, self).obj_dict(),
+            'blob_uuid': self._blob_uuid
+        }
+
+    # Data methods
+    def blob_uuid(self):
+        return self._blob_uuid
+
+
+class FetchBlobTask(BlobTask):
+    _name = 'blob_fetch'
