@@ -24,9 +24,6 @@ class FakeLibvirtConnection(object):
     def listDomainsID(self):
         return ['id1', 'id2', 'id3', 'id4', 'id5', 'id6']
 
-    def listDefinedDomains(self):
-        return ['sf:foo', 'sf:bar', 'docker', 'sf:crashed', 'sf:nofiles']
-
     def lookupByID(self, id):
         args = {
             'id1': ('sf:running', FakeLibvirt.VIR_DOMAIN_RUNNING),
@@ -39,6 +36,12 @@ class FakeLibvirtConnection(object):
 
         return FakeLibvirtDomain(*args.get(id))
 
+    def lookupByName(self, name):
+        return FakeLibvirtDomain(name, FakeLibvirt.VIR_DOMAIN_RUNNING)
+
+    def close(self):
+        pass
+
 
 class FakeLibvirtDomain(object):
     def __init__(self, name, state):
@@ -51,6 +54,9 @@ class FakeLibvirtDomain(object):
     def state(self):
         return [self._state, 1]
 
+    def UUIDString(self):
+        return 'fake_uuid'
+
 
 def fake_exists(path):
     if path == '/srv/shakenfist/instances/nofiles':
@@ -125,13 +131,13 @@ class CleanerTestCase(base.ShakenFistTestCase):
     @mock.patch('shakenfist.etcd.get', side_effect=fake_get)
     @mock.patch('shakenfist.db.get_lock')
     @mock.patch('shakenfist.node.Node.observe_this_node')
-    @mock.patch('shakenfist.db.add_event')
     @mock.patch('shakenfist.instance.Instance._db_get', side_effect=fake_instance_get)
     @mock.patch('shakenfist.etcd.put', side_effect=fake_put)
     @mock.patch('os.path.exists', side_effect=fake_exists)
     @mock.patch('time.time', return_value=7)
-    def test_update_power_states(self, mock_time, mock_exists, mock_put,
-                                 mock_get_instance, mock_event, mock_see,
+    @mock.patch('os.listdir', return_value=[])
+    def test_update_power_states(self, mock_listdir, mock_time, mock_exists,
+                                 mock_put, mock_get_instance, mock_see,
                                  mock_lock, mock_etcd_get, mock_error,
                                  mock_state_value, mock_state):
         mock_state_value.return_value = 'created'
@@ -160,9 +166,5 @@ class CleanerTestCase(base.ShakenFistTestCase):
                 ('attribute/instance', 'paused',
                  'power_state', {'power_state': 'paused'}),
                 ('attribute/instance', 'suspended',
-                 'power_state', {'power_state': 'paused'}),
-                ('attribute/instance', 'foo',
-                 'power_state', {'power_state': 'off'}),
-                ('attribute/instance', 'bar',
-                 'power_state', {'power_state': 'off'})
+                 'power_state', {'power_state': 'paused'})
             ], result)
