@@ -6,9 +6,11 @@ import six
 import testtools
 from pydantic import AnyHttpUrl, IPvAnyAddress
 
-from shakenfist import dhcp
-from shakenfist.ipmanager import IPManager
 from shakenfist.config import BaseSettings
+from shakenfist.ipmanager import IPManager
+from shakenfist import dhcp
+from shakenfist.instance import Instance
+from shakenfist.networkinterface import NetworkInterface
 
 TEST_DIR = os.path.dirname(os.path.abspath(__file__))
 
@@ -22,6 +24,7 @@ class FakeNetwork(object):
         self.netmask = '255.0.0.0'
         self.broadcast = '10.255.255.255'
         self.provide_nat = True
+        self.networkinterfaces = ['notauuid1', 'notauuid2']
 
 
 class FakeNetworkInterface(object):
@@ -128,30 +131,43 @@ class DHCPTestCase(testtools.TestCase):
         )
 
     @mock.patch('os.path.exists', return_value=True)
-    @mock.patch('shakenfist.networkinterface.NetworkInterfaces',
-                return_value=[
-                    FakeNetworkInterface({'uuid': 'notauuid1',
-                                          'instance_uuid': 'instuuid1',
-                                          'network_uuid': 'netuuid',
-                                          'macaddr': '1a:91:64:d2:15:39',
-                                          'ipv4': '127.0.0.5',
-                                          'order': 0}),
-                    FakeNetworkInterface({'uuid': 'notauuid2',
-                                          'instance_uuid': 'instuuid2',
-                                          'network_uuid': 'netuuid',
-                                          'macaddr': '1a:91:64:d2:15:40',
-                                          'ipv4': '127.0.0.6',
-                                          'order': 0})
+    @mock.patch('shakenfist.networkinterface.NetworkInterface._db_get',
+                side_effect=[
+                    {
+                        'uuid': 'notauuid1',
+                        'instance_uuid': 'instuuid1',
+                        'network_uuid': 'netuuid',
+                        'macaddr': '1a:91:64:d2:15:39',
+                        'model': 'virtio',
+                        'ipv4': '127.0.0.5',
+                        'order': 0,
+                        'version': NetworkInterface.current_version
+                    },
+                    {
+                        'uuid': 'notauuid2',
+                        'instance_uuid': 'instuuid2',
+                        'network_uuid': 'netuuid',
+                        'macaddr': '1a:91:64:d2:15:40',
+                        'model': 'virtio',
+                        'ipv4': '127.0.0.6',
+                        'order': 0,
+                        'version': NetworkInterface.current_version
+                    }
                 ])
     @mock.patch('shakenfist.instance.Instance._db_get',
                 side_effect=[
-                    {'uuid': 'instuuid1',
-                     'name': 'inst1',
-                     'disk_spec': [{'size': 4, 'base': 'foo'}]},
-                    {'uuid': 'instuuid2',
-                     'name': 'inst2',
-                     'disk_spec': [{'size': 4, 'base': 'foo'}]}
-                ])
+                    {
+                        'uuid': 'instuuid1',
+                        'name': 'inst1',
+                        'disk_spec': [{'size': 4, 'base': 'foo'}],
+                        'version': Instance.current_version
+                    },
+                    {
+                        'uuid': 'instuuid2',
+                        'name': 'inst2',
+                        'disk_spec': [{'size': 4, 'base': 'foo'}],
+                        'version': Instance.current_version
+                    }])
     def test_make_hosts(self, mock_instances, mock_interfaces, mock_exists):
         d = dhcp.DHCP(FakeNetwork(), 'eth0')
 
@@ -180,7 +196,9 @@ class DHCPTestCase(testtools.TestCase):
     @mock.patch('os.path.exists', return_value=True)
     @mock.patch('psutil.pid_exists', return_value=True)
     @mock.patch('os.kill')
-    def test_send_signal(self, mock_kill, mock_pid_exists, mock_path_exists):
+    @mock.patch('os.waitpid')
+    def test_send_signal(self, mock_waitpid, mock_kill, mock_pid_exists,
+                         mock_path_exists):
         d = dhcp.DHCP(FakeNetwork(), 'eth0')
 
         mock_open = mock.mock_open(read_data='424242')
