@@ -48,9 +48,6 @@ class BaseFakeObject(object):
     def unique_label(self):
         return ('instance', self.uuid)
 
-    def add_event(self, operation, phase, duration=None, msg=None):
-        pass
-
     def delete(self):
         pass
 
@@ -91,6 +88,9 @@ class FakeNetwork(BaseFakeObject):
     def remove_dhcp(self):
         pass
 
+    def networkinterfaces(self):
+        return []
+
 
 def _encode_key(key):
     return bcrypt.hashpw(key.encode('utf-8'), bcrypt.gensalt())
@@ -217,7 +217,8 @@ class AuthTestCase(base.ShakenFistTestCase):
                                 }))
         self.assertEqual(
             {
-                'error': "Bad Authorization header. Expected value 'Bearer <JWT>'",
+                'error': ("Missing 'Bearer' type in 'Authorization' header. Expected "
+                          "'Authorization: Bearer <JWT>'"),
                 'status': 401
             },
             _clean_traceback(resp.get_json()))
@@ -442,7 +443,7 @@ class ExternalApiGeneralTestCase(ExternalApiTestCase):
             resp.get_json())
 
     @mock.patch('shakenfist.instance.Instances', return_value=[])
-    @mock.patch('shakenfist.net.Networks',
+    @mock.patch('shakenfist.network.Networks',
                 return_value=[FakeNetwork(uuid='123', state=dbo.STATE_CREATED)])
     def test_delete_namespace_with_networks(self, mock_get_networks, mock_get_instances):
         resp = self.client.delete('/auth/namespaces/foo',
@@ -521,7 +522,8 @@ class ExternalApiGeneralTestCase(ExternalApiTestCase):
                                   headers={'Authorization': self.auth_token})
         self.assertEqual(None, resp.get_json())
         self.assertEqual(200, resp.status_code)
-        mock_md_put.assert_called_with('namespace', 'system', {'real': 'smart'})
+        mock_md_put.assert_called_with(
+            'namespace', 'system', {'real': 'smart'})
 
     @mock.patch('shakenfist.db.get_metadata', return_value={})
     @mock.patch('shakenfist.db.persist_metadata')
@@ -789,6 +791,24 @@ class ExternalApiGeneralTestCase(ExternalApiTestCase):
         self.assertEqual(404, resp.status_code)
 
 
+class ExternalApiNetworkInterfaceTestCase(ExternalApiTestCase):
+    def test_get_network_interface(self):
+        net = self.mock_etcd.create_network('barrynet')
+        nd = self.mock_etcd.generate_netdesc(net.uuid)
+        self.mock_etcd.create_network_interface(
+            uuid='88888888-1234-4321-1234-000000000001',
+            netdesc=nd,
+            instance_uuid='9999999-1234-4321-1234-000000000001')
+
+        # Get NetworkInterface
+        resp = self.client.get('/networks/barrynet/interfaces',
+                               headers={'Authorization': self.auth_token})
+        self.assertEqual(200, resp.status_code)
+        self.assertEqual('application/json', resp.content_type)
+        self.assertEqual('88888888-1234-4321-1234-000000000001',
+                         resp.get_json()[0].get('uuid'))
+
+
 class ExternalApiInstanceTestCase(ExternalApiTestCase):
     def setUp(self):
         super(ExternalApiInstanceTestCase, self).setUp()
@@ -922,9 +942,9 @@ class ExternalApiInstanceTestCase(ExternalApiTestCase):
         self.assertEqual(400, resp.status_code)
 
     @mock.patch('shakenfist.artifact.Artifact.from_url')
-    @mock.patch('shakenfist.net.Network._db_get_attribute',
+    @mock.patch('shakenfist.network.Network._db_get_attribute',
                 return_value={'value': dbo.STATE_CREATED, 'update_time': 2})
-    @mock.patch('shakenfist.net.Network.from_db',
+    @mock.patch('shakenfist.network.Network.from_db',
                 return_value=FakeNetwork(
                     uuid='87c15186-5f73-4947-a9fb-2183c4951efc',
                     vxid=1,
@@ -1033,10 +1053,6 @@ class ExternalApiNetworkTestCase(base.ShakenFistTestCase):
     def setUp(self):
         super(ExternalApiNetworkTestCase, self).setUp()
 
-        self.add_event = mock.patch('shakenfist.db.add_event')
-        self.mock_add_event = self.add_event.start()
-        self.addCleanup(self.add_event.stop)
-
         self.scheduler = mock.patch(
             'shakenfist.scheduler.Scheduler', FakeScheduler)
         self.mock_scheduler = self.scheduler.start()
@@ -1085,10 +1101,8 @@ class ExternalApiNetworkTestCase(base.ShakenFistTestCase):
             self.assertEqual(200, resp.status_code)
             self.auth_token = 'Bearer %s' % resp.get_json()['access_token']
 
-    @mock.patch('shakenfist.net.Network._db_get_attribute',
-                return_value={'value': dbo.STATE_CREATED, 'update_time': 2})
     @mock.patch('shakenfist.ipmanager.IPManager.from_db')
-    @mock.patch('shakenfist.net.Network.from_db',
+    @mock.patch('shakenfist.network.Network.from_db',
                 return_value=FakeNetwork(
                     uuid='30f6da44-look-i-am-uuid',
                     vxid=1,
@@ -1096,7 +1110,7 @@ class ExternalApiNetworkTestCase(base.ShakenFistTestCase):
                     name='bob',
                     netblock='10.10.0.0/24'
                 ))
-    @mock.patch('shakenfist.net.Networks',
+    @mock.patch('shakenfist.network.Networks',
                 return_value=[FakeNetwork(
                     uuid='30f6da44-look-i-am-uuid',
                     vxid=1,
@@ -1104,13 +1118,14 @@ class ExternalApiNetworkTestCase(base.ShakenFistTestCase):
                     name='bob',
                     netblock='10.10.0.0/24'
                 )])
-    @mock.patch('shakenfist.networkinterface.interfaces_for_network', return_value=[])
     @mock.patch('shakenfist.ipmanager.IPManager.from_db',
                 return_value=IPManager('uuid', '10.0.0.0/24'))
-    @mock.patch('shakenfist.net.Network.remove_dhcp')
-    @mock.patch('shakenfist.net.Network.delete_on_network_node')
-    @mock.patch('shakenfist.net.Network.delete_on_hypervisor')
-    @mock.patch('shakenfist.net.Network.state')
+    @mock.patch('shakenfist.network.Network.remove_dhcp')
+    @mock.patch('shakenfist.network.Network.delete_on_network_node')
+    @mock.patch('shakenfist.network.Network.delete_on_hypervisor')
+    @mock.patch('shakenfist.network.Network.state')
+    @mock.patch('shakenfist.network.Network.networkinterfaces',
+                new_callable=mock.PropertyMock)
     @mock.patch('shakenfist.etcd.put')
     @mock.patch('shakenfist.etcd.enqueue')
     @mock.patch('shakenfist.db.get_lock')
@@ -1118,16 +1133,16 @@ class ExternalApiNetworkTestCase(base.ShakenFistTestCase):
                                  mock_db_get_lock,
                                  mock_etcd_enqueue,
                                  mock_etcd_put,
+                                 mock_network_interfaces,
                                  mock_network_state,
                                  mock_delete_on_hypervisor,
                                  mock_delete_on_network_node,
                                  mock_remove_dhcp,
                                  mock_get_ipmanager,
-                                 mock_network_interfaces,
                                  mock_db_get_networks,
                                  mock_db_get_network,
-                                 mock_ipmanager_from_db,
-                                 mock_net_attribute):
+                                 mock_ipmanager_from_db):
+        mock_network_interfaces.return_value = []
 
         self.client = external_api.app.test_client()
         resp = self.client.delete('/networks',
@@ -1139,7 +1154,7 @@ class ExternalApiNetworkTestCase(base.ShakenFistTestCase):
         self.assertEqual(['30f6da44-look-i-am-uuid'], resp.get_json())
         self.assertEqual(200, resp.status_code)
 
-    @mock.patch('shakenfist.net.Network.from_db',
+    @mock.patch('shakenfist.network.Network.from_db',
                 return_value=FakeNetwork(
                     uuid='30f6da44-look-i-am-uuid',
                     vxid=1,
@@ -1150,10 +1165,9 @@ class ExternalApiNetworkTestCase(base.ShakenFistTestCase):
                 ))
     @mock.patch('shakenfist.etcd.get_all',
                 return_value=[(None, {'uuid': '30f6da44-look-i-am-uuid'})])
-    @mock.patch('shakenfist.networkinterface.interfaces_for_network', return_value=[])
     @mock.patch('shakenfist.ipmanager.IPManager.from_db',
                 return_value=IPManager('uuid', '10.0.0.0/24'))
-    @mock.patch('shakenfist.net.Network.remove_dhcp')
+    @mock.patch('shakenfist.network.Network.remove_dhcp')
     @mock.patch('shakenfist.etcd.put')
     @mock.patch('shakenfist.db.get_lock')
     def test_delete_all_networks_none_to_delete(self,
@@ -1161,7 +1175,6 @@ class ExternalApiNetworkTestCase(base.ShakenFistTestCase):
                                                 mock_etcd_put,
                                                 mock_remove_dhcp,
                                                 mock_get_ipmanager,
-                                                mock_network_interfaces,
                                                 mock_db_get_networks,
                                                 mock_db_get_network):
         resp = self.client.delete('/networks',
@@ -1177,11 +1190,6 @@ class ExternalApiNoNamespaceMockTestCase(base.ShakenFistTestCase):
     def setUp(self):
         super(ExternalApiNoNamespaceMockTestCase, self).setUp()
 
-        self.add_event = mock.patch(
-            'shakenfist.db.add_event')
-        self.mock_add_event = self.add_event.start()
-        self.addCleanup(self.add_event.stop)
-
         self.scheduler = mock.patch(
             'shakenfist.scheduler.Scheduler', FakeScheduler)
         self.mock_scheduler = self.scheduler.start()
