@@ -53,11 +53,15 @@ class VirtMetaTestCase(base.ShakenFistTestCase):
             NODE_NAME="node01",
         )
 
-        self.config = mock.patch('shakenfist.instance.config',
-                                 fake_config)
+        self.config = mock.patch('shakenfist.instance.config', fake_config)
         self.mock_config = self.config.start()
         self.addCleanup(self.config.stop)
 
+        self.gmov = mock.patch(
+            'shakenfist.instance.gmov', return_value=6)
+        self.mock_gmov = self.gmov.start()
+        self.addCleanup(self.gmov.stop)
+
     @mock.patch('shakenfist.etcd.get',
                 return_value={
                     'uuid': 'uuid42',
@@ -96,14 +100,11 @@ class VirtMetaTestCase(base.ShakenFistTestCase):
         self.assertEqual(
             ('attribute/instance', 'uuid42', 'state',
              State(instance.Instance.STATE_INITIAL, 1234)),
-            mock_put.mock_calls[1][1])
-        self.assertEqual(
-            ('attribute/instance', 'uuid42', 'error', {'message': None}),
-            mock_put.mock_calls[3][1])
+            mock_put.mock_calls[0][1])
         self.assertEqual(
             ('attribute/instance', 'uuid42',
              'power_state', {'power_state': instance.Instance.STATE_INITIAL}),
-            mock_put.mock_calls[4][1])
+            mock_put.mock_calls[1][1])
 
         self.assertEqual(
             ('instance', None, 'uuid42',
@@ -118,12 +119,13 @@ class VirtMetaTestCase(base.ShakenFistTestCase):
                  'ssh_key': 'sshkey',
                  'user_data': 'userdata',
                  'uuid': 'uuid42',
-                 'version': 6,
+                 'version': 7,
                  'video': {'memory': 16384, 'model': 'cirrus'},
                  'uefi': False,
                  'configdrive': 'openstack-disk',
                  'nvram_template': None,
-                 'secure_boot': False
+                 'secure_boot': False,
+                 'side_channels': None
              }),
             mock_create.mock_calls[0][1])
 
@@ -156,7 +158,7 @@ class VirtMetaTestCase(base.ShakenFistTestCase):
         self.assertEqual('sshkey', inst.ssh_key)
         self.assertEqual('userdata', inst.user_data)
         self.assertEqual('uuid42', inst.uuid)
-        self.assertEqual(6, inst.version)
+        self.assertEqual(7, inst.version)
         self.assertEqual('openstack-disk', inst.configdrive)
         self.assertEqual(None, inst.nvram_template)
         self.assertEqual(False, inst.secure_boot)
@@ -186,6 +188,11 @@ class InstanceTestCase(base.ShakenFistTestCase):
         self.mock_put = self.put.start()
         self.addCleanup(self.put.stop)
 
+        self.gmov = mock.patch(
+            'shakenfist.instance.gmov', return_value=6)
+        self.mock_gmov = self.gmov.start()
+        self.addCleanup(self.gmov.stop)
+
     @mock.patch('shakenfist.instance.Instance._db_create')
     @mock.patch('shakenfist.instance.Instance._db_get',
                 return_value={
@@ -412,7 +419,7 @@ class InstanceTestCase(base.ShakenFistTestCase):
                         'order': 1
                     })
                 ])
-    @mock.patch('shakenfist.net.Network.from_db',
+    @mock.patch('shakenfist.network.Network.from_db',
                 return_value=FakeNetwork())
     @mock.patch('shakenfist.baseobject.DatabaseBackedObject.state',
                 new_callable=mock.PropertyMock)
@@ -602,6 +609,14 @@ GET_ALL_INSTANCES = [
 
 
 class InstancesTestCase(base.ShakenFistTestCase):
+    def setUp(self):
+        super(InstancesTestCase, self).setUp()
+
+        self.gmov = mock.patch(
+            'shakenfist.instance.gmov', return_value=6)
+        self.mock_gmov = self.gmov.start()
+        self.addCleanup(self.gmov.stop)
+
     @mock.patch('shakenfist.etcd.get_all', return_value=GET_ALL_INSTANCES)
     def test_base_iteration(self, mock_get_all):
         uuids = []
